# -*- coding: utf-8 -*-
from Products.MemcachedManager.MemcachedManager import manage_addMemcachedManager

CACHEAUTHID = "MemcachedAuth"


def post_install(context):
    """Post install script"""
    if context.readDataFile("cpskincaching_default.txt") is None:
        return
    portal = context.getSite()
    setupMemcachedManager(portal)


def setupMemcachedManager(portal):
    if CACHEAUTHID not in portal.objectIds():
        manage_addMemcachedManager(portal, CACHEAUTHID)
    cacheMgr = getattr(portal, CACHEAUTHID)
    settings = {
        "request_vars": ["AUTHENTICATED_USER"],
        "servers": ("172.17.0.1:11211",),
        "mirrors": (),
        "max_age": 900,
        "debug": 0,
    }
    cacheMgr.manage_editProps("Memcached caching for auth", settings)



# -*- coding: utf-8 -*-
import logging
import os

from cpskin.caching.patch import isCachePurgingEnabled
from plone.cachepurging.hooks import KEY
from plone.cachepurging.interfaces import IPurger
from plone.cachepurging.utils import getURLsToPurge
from plone.registry.interfaces import IRegistry
from zope.annotation.interfaces import IAnnotations
from zope.component import adapter, queryUtility
from ZPublisher.interfaces import IPubSuccess

logger = logging.getLogger("cpskin.caching")


@adapter(IPubSuccess)
def purge(event):
    """
    Sends Purge requests based to proxies based on configuration
    defined in environment variables
    Code based on plone.cachepuring.hooks
    """
    request = event.request

    annotations = IAnnotations(request, None)
    if annotations is None:
        return

    paths = annotations.get(KEY, None)
    if paths is None:
        return

    registry = queryUtility(IRegistry)
    if registry is None:
        return

    if not isCachePurgingEnabled(registry=registry):
        return

    purger = queryUtility(IPurger)
    if purger is None:
        return

    proxies_url = getCachingConfiguration()
    if proxies_url:
        proxies_url = proxies_url.split(" ")
    else:
        return

    for path in paths:
        for url in getURLsToPurge(path, proxies_url):
            logger.debug("purging " + url)
            purger.purgeAsync(url)


def getCachingConfiguration():
    caching_servers = os.environ.get("CACHING_SERVERS", None)
    return caching_servers



# -*- coding: utf-8 -*-
"""Setup tests for this package."""
import unittest

from cpskin.caching.patch import isCachePurgingEnabled
from cpskin.caching.testing import CPSKIN_CACHING_INTEGRATION_TESTING
from plone import api
from plone.app.testing import quickInstallProduct


class TestSetup(unittest.TestCase):
    """Test that cpskin.caching is properly installed."""

    layer = CPSKIN_CACHING_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer["portal"]
        self.installer = api.portal.get_tool("portal_quickinstaller")

    def test_product_installed(self):
        """Test if cpskin.caching is installed."""
        self.assertTrue(self.installer.isProductInstalled("cpskin.caching"))

    def test_browserlayer(self):
        """Test that ICpskinCachingLayer is registered."""
        from cpskin.caching.interfaces import ICpskinCachingLayer
        from plone.browserlayer import utils

        self.assertIn(ICpskinCachingLayer, utils.registered_layers())

    def testReinstall(self):
        portal = self.layer["portal"]
        quickInstallProduct(portal, "kuleuven.caching")
        quickInstallProduct(portal, "kuleuven.caching")

    def testIsCachingEnabled(self):
        self.assertEqual(isCachePurgingEnabled(), True)



# -*- coding: utf-8 -*-
import cpskin.caching
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import (
    PLONE_FIXTURE,
    FunctionalTesting,
    IntegrationTesting,
    PloneSandboxLayer,
    applyProfile,
)
from plone.testing import z2


class CpskinCachingLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        self.loadZCML(package=cpskin.caching)

    def setUpPloneSite(self, portal):
        applyProfile(portal, "cpskin.caching:default")


CPSKIN_CACHING_FIXTURE = CpskinCachingLayer()


CPSKIN_CACHING_INTEGRATION_TESTING = IntegrationTesting(
    bases=(CPSKIN_CACHING_FIXTURE,), name="CpskinCachingLayer:IntegrationTesting"
)


CPSKIN_CACHING_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(CPSKIN_CACHING_FIXTURE,), name="CpskinCachingLayer:FunctionalTesting"
)


CPSKIN_CACHING_ACCEPTANCE_TESTING = FunctionalTesting(
    bases=(CPSKIN_CACHING_FIXTURE, REMOTE_LIBRARY_BUNDLE_FIXTURE, z2.ZSERVER_FIXTURE),
    name="CpskinCachingLayer:AcceptanceTesting",
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import unittest

from cpskin.caching.interfaces import ICpskinCachingLayer
from cpskin.caching.rewrite import CPSkinRewriter
from cpskin.caching.testing import CPSKIN_CACHING_INTEGRATION_TESTING
from cpskin.minisite.minisite import MinisiteConfig, decorateRequest
from plone.app.testing import TEST_USER_ID, setRoles
from zope.interface import directlyProvides


class TestCpskinRewriter(unittest.TestCase):
    layer = CPSKIN_CACHING_INTEGRATION_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.app.REQUEST
        directlyProvides(self.request, ICpskinCachingLayer)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.request = self.app.REQUEST
        self._prepareVHMRequest("/")
        self.rewriter = CPSkinRewriter(self.request)

    def tearDown(self):
        if os.environ.get("DOMAINS"):
            del os.environ["DOMAINS"]

    def _prepareVHMRequest(
        self, path, domain="imio.be", root="/plone", prefix="", protocol="http"
    ):
        translatedPrefix = "/".join(["_vh_%s" % p for p in prefix.split("/")])

        self.request["URL"] = "%s://%s%s%s" % (protocol, domain, prefix, path)
        self.request["ACTUAL_URL"] = "%s://%s%s%s" % (protocol, domain, prefix, path)
        self.request["SERVER_URL"] = "%s://%s" % (protocol, domain)
        self.request["PATH_INFO"] = (
            "/VirtualHostBase/%s/%s:80%s/VirtualHostRoot%s%s"
            % (protocol, domain, root, translatedPrefix, path)
        )
        self.request["VIRTUAL_URL"] = "%s://%s%s" % (protocol, domain, path)

        if prefix:
            self.request["VIRTUAL_URL_PARTS"] = (
                "%s://%s" % (protocol, domain),
                prefix[1:],
                path[1:],
            )
        else:
            self.request["VIRTUAL_URL_PARTS"] = (
                "%s://%s" % (protocol, domain),
                path[1:],
            )

        self.request["VirtualRootPhysicalPath"] = tuple(root.split("/"))

    def test_purge_paths_no_minisite(self):
        self.assertEqual(
            self.rewriter("/foo"),
            ["/VirtualHostBase/http/imio.be:80/plone/VirtualHostRoot/foo"],
        )

    def test_purge_paths_with_domains_no_minisite(self):
        os.environ["DOMAINS"] = "https://communesplone.be http://imio.be"
        self.assertListEqual(
            self.rewriter("/foo"),
            [
                "/VirtualHostBase/https/communesplone.be:443/plone/VirtualHostRoot/foo",
                "/VirtualHostBase/http/imio.be:80/plone/VirtualHostRoot/foo",
            ],
        )

    def test_purge_paths_with_minisite(self):
        minisite = MinisiteConfig(
            main_portal_url="http://localhost",
            minisite_url="https://loisirs.imio.be",
            search_path="/plone/test_minisite",
            filename="minisite_config.txt",
        )
        decorateRequest(self.request, minisite)
        self.assertListEqual(
            self.rewriter("/foo"),
            [
                "/VirtualHostBase/https/loisirs.imio.be:443/plone/VirtualHostRoot/foo",
                "/VirtualHostBase/http/imio.be:80/plone/VirtualHostRoot/foo",
            ],
        )

    def test_set_protocol_on_domain(self):
        self.assertEqual(
            self.rewriter.set_protocol_on_domain("imio.be"), "http://imio.be:80"
        )
        self.assertEqual(
            self.rewriter.set_protocol_on_domain("imio.be:443"), "https://imio.be:443"
        )
        self._prepareVHMRequest("/", protocol="https")
        self.assertEqual(
            self.rewriter.set_protocol_on_domain("bla.imio.be"),
            "https://bla.imio.be:443",
        )
        self._prepareVHMRequest("/", protocol="http")
        self.assertEqual(
            self.rewriter.set_protocol_on_domain("https://communesplone.be"),
            "https://communesplone.be:443",
        )
        self.assertEqual(
            self.rewriter.set_protocol_on_domain("https://bla.imio.be"),
            "https://bla.imio.be:443",
        )

    def test_raise_parsing_domain(self):
        with self.assertRaises(ValueError):
            self.rewriter.set_protocol_on_domain("")



# -*- coding: utf-8 -*-
# from plone import api


def reload_registry_xml(context):
    # kid = 'plone.cachepurging.interfaces.ICachePurgingSettings.virtualHosting'
    # api.portal.set_registry_record(kid, True)
    context.runImportStepFromProfile(
        'profile-cpskin.caching:default', 'plone.app.registry')



# -*- coding: utf-8 -*-
from plone.cachepurging import utils
from plone.cachepurging.interfaces import ICachePurgingSettings
from plone.registry.interfaces import IRegistry
from zope.component import queryUtility


def isCachePurgingEnabled(registry=None):
    """
    we redefine this function because we don't want to check if there are
    cachingProxies in the registry
    """
    if registry is None:
        registry = queryUtility(IRegistry)
    if registry is None:
        return False
    settings = registry.forInterface(ICachePurgingSettings, check=False)
    return settings.enabled


utils.isCachePurgingEnabled = isCachePurgingEnabled



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import unittest

import zope.component.testing
from cpskin.caching.hooks import purge
from cpskin.caching.testing import CPSKIN_CACHING_INTEGRATION_TESTING  # noqa
from plone.cachepurging.interfaces import ICachePurgingSettings, IPurger
from plone.cachepurging.tests.test_hooks import FauxRequest
from plone.registry import Registry
from plone.registry.fieldfactory import persistentFieldAdapter
from plone.registry.interfaces import IRegistry
from zope.annotation.attribute import AttributeAnnotations
from zope.annotation.interfaces import IAnnotations, IAttributeAnnotatable
from zope.component import provideAdapter, provideHandler, provideUtility
from zope.event import notify
from zope.interface import alsoProvides, implements
from ZPublisher.pubevents import PubSuccess


class TestPurgeHandler(unittest.TestCase):
    def setUp(self):
        provideAdapter(AttributeAnnotations)
        provideAdapter(persistentFieldAdapter)
        provideHandler(purge)

    def tearDown(self):
        zope.component.testing.tearDown()

    def test_purge_no_config(self):
        request = FauxRequest()
        alsoProvides(request, IAttributeAnnotatable)

        IAnnotations(request)["plone.cachepurging.urls"] = set(["/foo", "/bar"])

        registry = Registry()
        registry.registerInterface(ICachePurgingSettings)
        provideUtility(registry, IRegistry)
        settings = registry.forInterface(ICachePurgingSettings)
        settings.enabled = True

        class FauxPurger(object):
            implements(IPurger)

            def __init__(self):
                self.purged = []

            def purgeAsync(self, url, httpVerb="PURGE"):
                self.purged.append(url)

        purger = FauxPurger()
        provideUtility(purger)

        notify(PubSuccess(request))

        self.assertEquals([], purger.purged)

    def test_purge_with_config(self):
        request = FauxRequest()
        alsoProvides(request, IAttributeAnnotatable)

        IAnnotations(request)["plone.cachepurging.urls"] = set(["/foo", "/bar"])

        registry = Registry()
        registry.registerInterface(ICachePurgingSettings)
        provideUtility(registry, IRegistry)

        settings = registry.forInterface(ICachePurgingSettings)
        settings.enabled = True

        class FauxPurger(object):
            implements(IPurger)

            def __init__(self):
                self.purged = []

            def purgeAsync(self, url, httpVerb="PURGE"):
                self.purged.append(url)

        purger = FauxPurger()
        provideUtility(purger)
        os.environ["CACHING_SERVERS"] = "http://localhost:1234 http://10.0.100.1:1235"
        notify(PubSuccess(request))

        self.assertEquals(
            [
                "http://localhost:1234/foo",
                "http://10.0.100.1:1235/foo",
                "http://localhost:1234/bar",
                "http://10.0.100.1:1235/bar",
            ],
            purger.purged,
        )


def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)



# -*- coding: utf-8 -*-
"""Installer for the cpskin.caching package."""

from setuptools import find_packages, setup

long_description = (
    open("README.rst").read()
    + "\n"
    + "Contributors\n"
    + "============\n"
    + "\n"
    + open("CONTRIBUTORS.rst").read()
    + "\n"
    + open("CHANGES.rst").read()
    + "\n"
)


setup(
    name="cpskin.caching",
    version="1.1.3",
    description="Caching configuration for cpskin",
    long_description=long_description,
    # Get more from https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: 4.3",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
    ],
    keywords="Python Plone",
    author="Jean-Francois Roche",
    author_email="jfroche@affinitic.be",
    url="https://pypi.python.org/pypi/cpskin.caching",
    license="GPL version 2",
    packages=find_packages("src", exclude=["ez_setup"]),
    namespace_packages=["cpskin"],
    package_dir={"": "src"},
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        "plone.api",
        "setuptools",
        "z3c.jbot",
        "Products.MemcachedManager",
        "z3c.unconfigure",
        "collective.monkeypatcher>=1.0.1",
        "python-memcached",
        "cpskin.minisite",
    ],
    extras_require={
        "test": [
            "plone.app.testing",
            "plone.app.contenttypes",
            "plone.app.robotframework[debug]",
        ]
    },
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    """,
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import re
import urlparse

from cpskin.caching.interfaces import ICpskinCachingLayer
from plone.cachepurging.interfaces import ICachePurgingSettings, IPurgePathRewriter
from plone.registry.interfaces import IRegistry
from zope.component import adapts, queryUtility
from zope.interface import implements


class CPSkinRewriter(object):
    """Default rewriter, which is aware of virtual hosting and minisites
    """

    implements(IPurgePathRewriter)
    adapts(ICpskinCachingLayer)

    def __init__(self, request):
        self.request = request

    def set_protocol_on_domain(self, domain):
        portSchemeMap = {"http": 80, "https": 443, "80": "http", "443": "https"}
        parsedDomain = re.search(
            "(?:(?P<scheme>http?|https?)://)?(?P<host>[^:/ ]+).?(?P<port>[0-9]*).*",
            domain,
        )
        if parsedDomain is not None:
            parsedDomain = parsedDomain.groupdict()
        else:
            raise ValueError("Unable to parse domain: {}".format(domain))
        currentScheme = parsedDomain.get("scheme")
        if not currentScheme and parsedDomain.get("port"):
            currentScheme = portSchemeMap[parsedDomain.get("port")]
        if not currentScheme:
            currentScheme = urlparse.urlparse(self.request.URL).scheme
        hostname = parsedDomain.get("host")
        if hostname is None:
            hostname = domain
        return "%s://%s:%s" % (currentScheme, hostname, portSchemeMap[currentScheme])

    def __call__(self, path):
        request = self.request
        # No rewriting necessary
        virtualURL = request.get("VIRTUAL_URL", None)
        if virtualURL is None:
            return [path]

        registry = queryUtility(IRegistry)
        if registry is None:
            return [path]

        settings = registry.forInterface(ICachePurgingSettings, check=False)

        virtualHosting = settings.virtualHosting

        # We don't want to rewrite
        if not virtualHosting:
            return [path]

        # We need to reconstruct VHM URLs for each of the domains
        virtualUrlParts = request.get("VIRTUAL_URL_PARTS")
        virtualRootPhysicalPath = request.get("VirtualRootPhysicalPath")
        # Make sure request is compliant
        if (
            not virtualUrlParts
            or not virtualRootPhysicalPath
            or not isinstance(virtualUrlParts, (list, tuple))
            or not isinstance(virtualRootPhysicalPath, (list, tuple))
            or len(virtualUrlParts) < 2
            or len(virtualUrlParts) > 3
        ):
            return [path]

        def get_env_domains():
            domains = os.environ.get("DOMAINS")
            if domains:
                return domains.split(" ")
            else:
                return []

        environ_domains = set(get_env_domains())
        registry_domains = set(settings.domains)
        domains = environ_domains.union(registry_domains)
        if not domains:
            domains = set([virtualUrlParts[0]])

        # Minisites
        minisite = request.get("cpskin_minisite", None)
        if minisite and minisite.is_minisite:
            domains.add(request.cpskin_minisite.minisite_url)

        # Virtual root, e.g. /Plone. Clear if we don't have any virtual root
        virtualRoot = "/".join(virtualRootPhysicalPath)
        if virtualRoot == "/":
            virtualRoot = ""

        # Prefix, e.g. /_vh_foo/_vh_bar. Clear if we don't have any.
        pathPrefix = len(virtualUrlParts) == 3 and virtualUrlParts[1] or ""
        if pathPrefix:
            pathPrefix = "/" + "/".join(["_vh_%s" % p for p in pathPrefix.split("/")])

        # Path, e.g. /front-page
        if not path.startswith("/"):
            path = "/" + path

        paths = []
        for domain in domains:
            domain = self.set_protocol_on_domain(domain)
            scheme, host = urlparse.urlparse(domain)[:2]
            paths.append(
                "/VirtualHostBase/%(scheme)s/%(host)s%(root)s/VirtualHostRoot%(prefix)s%(path)s"
                % {
                    "scheme": scheme,
                    "host": host,
                    "root": virtualRoot,
                    "prefix": pathPrefix,
                    "path": path,
                }
            )
        return paths



# -*- coding: utf-8 -*-
from plone import api
from plone.behavior.interfaces import IBehaviorAssignable
from plone.dexterity.interfaces import IDexterityFTI
from plone.dexterity.interfaces import IDexteritySchema
from plone.namedfile.interfaces import INamedBlobFileField
from plone.namedfile.interfaces import INamedImageField
from z3c.caching.interfaces import IPurgePaths
from zope.component import adapter
from zope.component import getUtility
from zope.interface import implementer
from zope.schema import getFieldsInOrder

import six


@implementer(IPurgePaths)
@adapter(IDexteritySchema)
class ScalesPurgePaths(object):
    """Paths to purge for Dexterity object fields
    """
    def __init__(self, context):
        self.context = context

    def getScales(self):
        scale_util = api.content.get_view(
            'images',
            self.context,
            self.context.REQUEST)
        return scale_util.getAvailableSizes().keys()

        # registry = getUtility(IRegistry)
        # reg_list = registry['plone.allowed_sizes']
        # sizes = [i.split(' ', 1)[0] for i in reg_list]
        # sizes.append('download')
        # return sizes

    def getRelativePaths(self):
        prefix = '/' + self.context.virtual_url_path()

        def fieldFilter():
            portal_type = self.context.getPortalTypeName()
            fti = getUtility(IDexterityFTI, name=portal_type)
            schema = fti.lookupSchema()
            fields = getFieldsInOrder(schema)
            assignable = IBehaviorAssignable(self.context, None)
            for behavior in assignable.enumerateBehaviors():
                if behavior.marker:
                    new_fields = getFieldsInOrder(behavior.marker)
                    if len(new_fields) > 0:
                        fields = fields + new_fields

            obj_fields = []
            for key, value in fields:
                is_image = INamedImageField.providedBy(value)
                is_file = INamedBlobFileField.providedBy(value)
                if is_image or is_file:
                    obj_fields.append(value)
            return obj_fields

        for item in fieldFilter():
            field = item.getName()
            value = item.get(self.context)
            if not value:
                continue

            if INamedImageField.providedBy(item):
                for size in self.getScales():
                    yield '{0}/@@images/{1}/{2}'.format(prefix, field, size,)
                    yield '{0}/@@download/{1}'.format(prefix, field)
            else:
                filename = value.filename
                if six.PY2 and isinstance(filename, six.text_type):
                    filename = filename.encode('utf-8')
                yield '{0}/view/{1}.{2}/@@download/{3}'.format(
                    prefix, '++widget++form.widgets', field, filename)
                yield '{0}/@@download/{1}/{2}'.format(prefix, field, filename)

    def getAbsolutePaths(self):
        return []
